@@ -138,19 +138,19 @@ class OptimizerSwapper(object):
 
         self.optimizer = optimizer
 
-        # Swap buffer management
-        self.largest_numel = largest_numel
-        self.dtype = dtype
-        self.swap_buffer_manager = SwapBufferManager(
-            num_elems=largest_numel,
-            count=swap_config[OFFLOAD_OPTIMIZER_BUFFER_COUNT],
-            dtype=dtype)
-
         # Read/Write alignment for each thread during Intra-request parallelism
         self.min_aio_bytes = max(MIN_AIO_BYTES, aio_config[AIO_BLOCK_SIZE])
         self.aligned_bytes = AIO_ALIGNED_BYTES * aio_config[AIO_THREAD_COUNT]
         self.numel_alignment = self.aligned_bytes // self.swap_element_size
 
+        # Swap buffer management
+        self.largest_numel = self._io_aligned_numel(largest_numel)
+        self.dtype = dtype
+        self.swap_buffer_manager = SwapBufferManager(
+            num_elems=self.largest_numel,
+            count=swap_config[OFFLOAD_OPTIMIZER_BUFFER_COUNT],
+            dtype=dtype)
+
         # Timers
         self.timers = timers
         self.timer_names = set()
